<?php
/*
Plugin Name: Wordpress Logging Service
Version: 1.4.9
Description: Provides a simple API for storing miscellaneous log entries and their overview in admin area.
Author: Zaantar
Author URI: http://zaantar.eu
License: GPL2
*/
?><?php

/*  Copyright 2011 Zaantar (email: zaantar@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

?><?php

// TODO exporty logů, custom fields(?), automatické promazávání (nastavení # záznamů/čas), pro každý log datum "last reviewed"

require_once plugin_dir_path( __FILE__ ).'includes/admin-overview.php';
require_once plugin_dir_path( __FILE__ ).'includes/options.php';

/*****************************************************************************\
		I18N
\*****************************************************************************/


define( 'WLS_TEXTDOMAIN', 'wordpress-logging-service' );

add_action( 'init', 'wls_load_textdomain' );

function wls_load_textdomain() {
	$plugin_dir = basename(dirname(__FILE__));
	load_plugin_textdomain( WLS_TEXTDOMAIN, false, $plugin_dir.'/languages' );
}


/*****************************************************************************\
		DATABASE
\*****************************************************************************/


define( 'WLS_VERSION', '1.4.2' );


register_activation_hook( __FILE__, 'wls_plugin_activation' );


function wls_logs_table() {
	global $wpdb;
	return $wpdb->base_prefix.'wls_logs';
}


function wls_entries_table() {
	global $wpdb;
	return $wpdb->base_prefix.'wls_entries';
}


function wls_plugin_activation() {
	global $wpdb;

	$logs_structure = 
        'CREATE TABLE IF NOT EXISTS '.wls_logs_table().' (
            id INT NOT NULL AUTO_INCREMENT,
            log_name VARCHAR(255),
            description LONGTEXT, 
            UNIQUE ( id ),
            PRIMARY KEY ( id )
        )';
        
    $wpdb->query( $wpdb->prepare( $logs_structure ) );

	$entries_structure = 
        'CREATE TABLE IF NOT EXISTS '.wls_entries_table().' (
            id INT NOT NULL AUTO_INCREMENT,
            log_id INT,
            blog_id BIGINT(20),
            date DATETIME,
            user_id BIGINT(20),
            category INT,
            text LONGTEXT,
            seen BOOL DEFAULT FALSE,
            UNIQUE ( id ),
            PRIMARY KEY ( id )
        )';
        
    $wpdb->query( $wpdb->prepare( $entries_structure ) );
    
    wls_set_version( WLS_VERSION );
}


function wls_get_log_id( $log_name ) {
	global $wpdb;
	$query = 'SELECT id
		FROM '.wls_logs_table().' 
		WHERE log_name LIKE \''.$log_name.'\'
		LIMIT 1';
	$log_id = $wpdb->get_var( $wpdb->prepare( $query ) );
	return $log_id;
}

function wls_get_logs( ) {
	global $wpdb;
	$query = 'SELECT *
		FROM '.wls_logs_table().'
		ORDER BY log_name ASC';
	return $wpdb->get_results( $wpdb->prepare( $query ) );
} 


function wls_get_log( $log_id ) {
	global $wpdb;
	$query = 'SELECT * 
		FROM '.wls_logs_table().'
		WHERE id = '.$log_id;
	return $wpdb->get_row( $wpdb->prepare( $query ) );
}


/**
 * @param $log_id log id or 'all' for all logs
 * @param $seen if true, will show all entries. if false, only unseen
 */
function wls_get_entries( $log_id = 'all', $from = 0, $to = 100, $seen = true, $order = 'DESC', $min_severity = 0,
	$what = 'entries' ) {	
	
	global $wpdb;
	
	$single = false;
	if( $what == 'entries' ) {
		$what = '*';
	} else if( $what == 'count' ) {
		$what = 'COUNT(*)';
		$single = true;
	}

	$cond = array();
	if( $log_id != 'all' ) {
		$cond[] = 'log_id = '.$log_id;
	}
	if( !$seen ) {
		$cond[] = 'seen = 0';
	}
	$cond[] = 'category >= '.$min_severity;
	
	if( count( $cond ) > 0 ) {
		$where = ' WHERE '.implode( ' AND ', $cond );
	} else {
		$where = '';
	}

	$query = 'SELECT '.$what.'
		FROM '.wls_entries_table().'
		'.$where.'
		ORDER BY date '.$order.', id '.$order.' 
		LIMIT '.$from.', '.$to;

	if( $single ) {	
		return $wpdb->get_var( $wpdb->prepare( $query ) );
	} else {
		return $wpdb->get_results( $wpdb->prepare( $query ) );
	}
}


/*function wls_get_unseen_entries( $from = 0, $to = 100, $order = 'ASC' ) {
	if( get_current_user_id() != 1 ) {
		return array();
	}
	global $wpdb;
	$query = 'SELECT *
		FROM '.wls_entries_table().'
		WHERE seen = 0
		ORDER BY date '.$order.', id '.$order.'
		LIMIT '.$from.', '.$to;
	$entries = $wpdb->get_results( $wpdb->prepare( $query ) );
	return $entries;
}*/


function wls_get_entry_count( $log_id ) {
	global $wpdb;
	$query = 'SELECT COUNT(*)
		FROM '.wls_entries_table().'
		WHERE log_id = '.$log_id;
	return $wpdb->get_var( $wpdb->prepare( $query ) );
}


function wls_get_unseen_entry_count( $min_category = 'default' ) {
	$settings = wls_get_settings();
	extract( $settings );
		
	if( get_current_user_id() != $wls_manager_id ) {
		return 0;
	}
	
	//echo '<br>'.$min_category.' ';
	
	if( !strcmp( $min_category, 'default' ) ) {
		echo ' DEFX ';
		$min_category = $def_severity_filter;
	}
	
	
	global $wpdb;
	$query = 'SELECT COUNT(*)
		FROM '.wls_entries_table().'
		WHERE seen = 0 AND category >= '.$min_category;
		
	//echo $query;
	return $wpdb->get_var( $wpdb->prepare( $query ) );
}


function wls_mark_seen( $entry_id ) {
	global $wpdb;
	$wpdb->update( wls_entries_table(), array( 'seen' => '1' ), array( 'id' => $entry_id ), '%d', '%d' );
}


/*****************************************************************************\
		SELF_LOG
\*****************************************************************************/


function wls_selflog( $message, $category = 1 ) {
	if( !wls_is_registered( 'wordpress-logging-service' ) ) {
		$filename = dirname(__FILE__).'/log.txt';
		$file = fopen( $filename, 'ä́' );
		fwrite( $file, $message );
		fclose( $file );
	} else {
		wls_simple_log( 'wordpress-logging-service', $message, $category );
	}
}


/*****************************************************************************\
		UPGRADING
\*****************************************************************************/


add_action( 'plugins_loaded', 'wls_upgrade_db_check' );

function wls_upgrade_db_check() {
	$version = wls_get_version();
	$vp = wls_get_version_parts( $version );
	$cvp = wls_get_version_parts( WLS_VERSION );
	if( ( $vp[0] >= 1 ) && ( $vp[1] >= 4 ) ) {
		return;
	} else {
		wls_selflog( 'Version has changed (from '.$version.' to '.WLS_VERSION.'), upgrade is neccessary.', 3 );

		wls_upgrade_from_first_to_1_4_0();

		$new_version = wls_get_version();
		wls_selflog( 'Current version is now '.$new_version.'.' );

		wls_upgrade_db_check();
	}
}

		
function wls_get_version_parts( $version ) {
	$parts = explode( '.', $version );
	if( count( $parts ) < 2 ) {
		$parts[1] = 0;
	}
	if( count( $parts ) < 3 ) {
		$parts[2] = 0;
	}
	if( !is_int( $parts[2] ) ) {
		$parts[2] = preg_replace ('/[^\d\s]/', '', $parts[2] );
	}
	return $parts;
}


function wls_set_version( $version ) {
	update_site_option( 'wls_version', $version );
}

function wls_get_version() {
	return get_site_option( 'wls_version', '1.3.0' );
}


function wls_upgrade_from_first_to_1_4_0() {
	wls_selflog( 'Performing database upgrade to version 1.4.0.' );
	
	// zaregistrujeme log sebe sama
	wls_register( 'wordpress-logging-service', _e( 'Log entries of Wordpress Logging Service', WLS_TEXTDOMAIN ) ); //'Systémové záznamy pluginu Wordpress Logging Service'
	
	// pridame sloupec 'seen'
	if ( !function_exists( 'maybe_add_column' ) ) {
		require_once(ABSPATH . 'wp-admin/install-helper.php');
	}
	maybe_add_column( wls_entries_table(), 'seen', 'ALTER TABLE '.wls_entries_table().' ADD COLUMN seen BOOL DEFAULT FALSE' );
	
	wls_set_version( '1.4.0' );
}

/*****************************************************************************\
		PUBLIC API
\*****************************************************************************/

define( 'WLS', true );

define( 'WLS_NOCATEGORY', 0 );
define( 'WLS_INFO', 1 );
define( 'WLS_NOTICE', 2 );
define( 'WLS_WARNING', 3 );
define( 'WLS_ERROR', 4 );
define( 'WLS_FATALERROR', 5 );

function wls_is_registered( $log_name ) {
	global $wpdb;
	$query = '
		SELECT COUNT(*)
		FROM '.wls_logs_table().' 
		WHERE log_name LIKE \''.$log_name.'\'';
	return ( $wpdb->get_var( $wpdb->prepare( $query ) ) > 0 );
}

function wls_register( $log_name, $description ) {
	global $wpdb;
	if( wls_is_registered( $log_name ) ) {
		return false;
	}
	$ok = $wpdb->insert( wls_logs_table(), 
		array( 'log_name' => $log_name, 'description' => $description ) ); 
	return ( $ok != false );
}

function wls_unregister( $log_name ) {
	global $wpdb;
	if( !wls_is_registered( $log_name ) ) {
		return false;
	}
	wls_clear( $log_name );
	$query = 'DELETE FROM '.wls_logs_table().'
		WHERE log_name LIKE \''.$log_name.'\'';
	return ( $wpdb->query( $wpdb->prepare( $query ) ) != FALSE );
}

function wls_clear( $log_name ) {
	global $wpdb;
	if( !wls_is_registered( $log_name ) ) {
		return false;
	}
	$query = 'DELETE FROM '.wls_entries_table().'
		WHERE log_id = '.wls_get_log_id( $log_name );
	return ( $wpdb->query( $wpdb->prepare( $query ) ) != FALSE );
}

function wls_log( $log_name, $text, $user_id, $date, $blog_id, $category = WLS_NOCATEGORY ) {
	if( !wls_is_registered( $log_name ) ) {
		return false;
	}
	$data = array( 
		'log_id' => wls_get_log_id( $log_name ), 
		'blog_id' => $blog_id,
		'date' => $date,
		'user_id' => $user_id,
		'text' => stripslashes( $text ),
		'category' => $category
	);
	
	$lastlog = wls_get_last_log();
	if( $lastlog == NULL ) {
		$ok = true;
		wls_update_last_log( $data );
	} else if( ( $lastlog['log_data']['log_id'] != $data['log_id'] )
		or ( $lastlog['log_data']['blog_id'] != $data['blog_id'] ) 
		or ( $lastlog['log_data']['user_id'] != $data['user_id'] )
		or ( $lastlog['log_data']['category'] != $data['category'] )
		or ( $lastlog['log_data']['text'] != $data['text'] ) ) {
		$ok = wls_flush_last_log();
		wls_update_last_log( $data );
	} else {
		$lastlog['last'] = $data['date'];
		$lastlog['count'] = $lastlog['count']+1;
		update_site_option( 'WLS_LAST_LOG', $lastlog );
		$ok = true;
	}
	return $ok;
}

function wls_simple_log( $log_name, $text, $category = WLS_NOCATEGORY ) {
	return wls_log( $log_name, $text, get_current_user_id(), date( 'c' ), get_current_blog_id(), $category );
}

/*****************************************************************************\
		REPEATING LOG TRIMMING
\*****************************************************************************/

define( 'WLS_LAST_LOG', 'wls_last_log' );


function wls_get_last_log() {
	$log = get_site_option( WLS_LAST_LOG, NULL );
	return $log;
}


function wls_flush_last_log() {
	$log = wls_get_last_log();
	if( $log == NULL ) {
		return true;
	}
	$data = $log['log_data'];
	if( $log['count'] > 1 ) {
		$data['text'] = '('.$log['count'].'x, last occurence @ '.$log['last'].'): '."\n".$data['text'];
	}
	//echo ' FLUSH ';
	$ok = wls_insert_log( $data );
	wls_clear_last_log();
	return $ok;
}


function wls_clear_last_log() {
	update_site_option( WLS_LAST_LOG, NULL );
}

function wls_update_last_log( $data ) {
	$log = array(
		'log_data' => $data,
		'last' => $data['date'],
		'count' => 1
	);
	update_site_option( WLS_LAST_LOG, $log );
}

function wls_overwrite_last_log( $data ) {
	wls_flush_last_log();
	wls_update_last_log( $data );
}

function wls_insert_log( $data ) {
	global $wpdb;
	$ok = $wpdb->insert( wls_entries_table(), $data, array( '%d', '%d', '%s', '%d', '%s', '%d' ) );
	return ( $ok != false );
}


?>
